@@ -1,8 +1,8 @@
-/*	$Id: configfile.c 16319 2009-07-22 20:19:38Z moritz $	*/
+/*	$Id$	*/
 /*
  *  ezstream - source client for Icecast with external en-/decoder support
  *  Copyright (C) 2003, 2004, 2005, 2006  Ed Zaleski <oddsock@oddsock.org>
- *  Copyright (C) 2007, 2009              Moritz Grimm <mdgrimm@gmx.net>
+ *  Copyright (C) 2007, 2009, 2015        Moritz Grimm <mgrimm@mrsserver.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -89,7 +89,8 @@ parseConfig(const char *fileName)
 	xmlNodePtr	 cur;
 	char		*ls_xmlContentPtr;
 	int		 program_set, reconnect_set, shuffle_set,
-			 streamOnce_set, svrinfopublic_set;
+			 streamOnce_set, svrinfopublic_set,
+			 refresh_set;
 	unsigned int	 config_error;
 
 	xmlLineNumbersDefault(1);
@@ -106,14 +107,17 @@ parseConfig(const char *fileName)
 		return (0);
 	}
 
-	memset(&ezConfig, '\000', sizeof(ezConfig));
+	memset(&ezConfig, 0, sizeof(ezConfig));
+	ezConfig.metadataRefreshInterval = -1;
 
 	config_error = 0;
 	program_set = 0;
 	reconnect_set = 0;
+	refresh_set = 0;
 	shuffle_set = 0;
 	streamOnce_set = 0;
 	svrinfopublic_set = 0;
+
 	for (cur = cur->xmlChildrenNode; cur != NULL; cur = cur->next) {
 		if (!xmlStrcmp(cur->name, (const xmlChar *)"url")) {
 			if (ezConfig.URL != NULL) {
@@ -230,6 +234,27 @@ parseConfig(const char *fileName)
 				}
 			}
 		}
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"metadata_refreshinterval")) {
+			if (refresh_set) {
+				printf("%s[%ld]: Error: Cannot have multiple <metadata_refreshinterval> elements\n",
+				       fileName, xmlGetLineNo(cur));
+				config_error++;
+				continue;
+			}
+			if (cur->xmlChildrenNode != NULL) {
+				const char *errstr;
+				ls_xmlContentPtr = (char *)xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+				ezConfig.metadataRefreshInterval = (unsigned int)strtonum(ls_xmlContentPtr, -1LL, (long long)INT_MAX, &errstr);
+				if (errstr) {
+					printf("%s[%ld]: Error: In <metadata_refreshinterval>: '%s' is %s\n",
+					       fileName, xmlGetLineNo(cur), ls_xmlContentPtr, errstr);
+					config_error++;
+					continue;
+				}
+				xmlFree(ls_xmlContentPtr);
+				refresh_set = 1;
+			}
+		}
 		if (!xmlStrcmp(cur->name, (const xmlChar *)"playlist_program")) {
 			if (program_set) {
 				printf("%s[%ld]: Error: Cannot have multiple <playlist_program> elements\n",
