@@ -1,8 +1,8 @@
-/*	$Id: ezstream.c 16537 2009-08-30 21:55:24Z moritz $	*/
+/*	$Id$	*/
 /*
  *  ezstream - source client for Icecast with external en-/decoder support
  *  Copyright (C) 2003, 2004, 2005, 2006  Ed Zaleski <oddsock@oddsock.org>
- *  Copyright (C) 2007, 2009              Moritz Grimm <mdgrimm@gmx.net>
+ *  Copyright (C) 2007, 2009, 2015        Moritz Grimm <mgrimm@mrsserver.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -50,6 +50,7 @@ char			*__progname;
 #endif /* HAVE___PROGNAME */
 
 int			 nFlag;
+int			 mFlag;
 int			 qFlag;
 int			 sFlag;
 int			 vFlag;
@@ -89,8 +90,8 @@ typedef struct tag_ID3Tag {
 } ID3Tag;
 
 int		urlParse(const char *, char **, unsigned short *, char **);
-void		replaceString(const char *, char *, size_t, const char *,
-			      const char *);
+char *		shellQuote(const char *);
+char *		replaceString(const char *, const char *, const char *);
 char *		buildCommandString(const char *, const char *, metadata_t *);
 char *		getMetadataString(const char *, metadata_t *);
 metadata_t *	getMetadata(const char *);
@@ -196,25 +197,67 @@ urlParse(const char *url, char **hostname, unsigned short *port,
 	return (1);
 }
 
-void
-replaceString(const char *source, char *dest, size_t size,
-	      const char *from, const char *to)
-{
-	const char	*p1 = source;
-	const char	*p2;
+#define SHELLQUOTE_INLEN_MAX	8191UL
 
+char *
+shellQuote(const char *in)
+{
+	char		*out, *out_p;
+	size_t		 out_len;
+	const char	*in_p;
+
+	out_len = (strlen(in) > SHELLQUOTE_INLEN_MAX)
+	    ? SHELLQUOTE_INLEN_MAX
+	    : strlen(in);
+	out_len = out_len * 2 + 2;
+	out = xcalloc(out_len + 1, sizeof(char));
+
+	out_p = out;
+	in_p = in;
+
+	*out_p++ = '\'';
+	out_len--;
+	while (*in_p && out_len > 2) {
+		switch (*in_p) {
+		case '\'':
+		case '\\':
+			*out_p++ = '\\';
+			out_len--;
+			break;
+		default:
+			break;
+		}
+		*out_p++ = *in_p++;
+		out_len--;
+	}
+	*out_p++ = '\'';
+
+	return (out);
+}
+
+char *
+replaceString(const char *source, const char *from, const char *to)
+{
+	char		*to_quoted, *dest;
+	size_t		 dest_size;
+	const char	*p1, *p2;
+
+	to_quoted = shellQuote(to);
+	dest_size = strlen(source) + strlen(to_quoted) + 1;
+	dest = xcalloc(dest_size, sizeof(char));
+
+	p1 = source;
 	p2 = strstr(p1, from);
 	if (p2 != NULL) {
-		if ((unsigned int)(p2 - p1) >= size) {
-			printf("%s: replaceString(): Internal error: p2 - p1 >= size\n",
-			       __progname);
-			abort();
-		}
 		strncat(dest, p1, (size_t)(p2 - p1));
-		strlcat(dest, to, size);
+		strlcat(dest, to_quoted, dest_size);
 		p1 = p2 + strlen(from);
 	}
-	strlcat(dest, p1, size);
+	strlcat(dest, p1, dest_size);
+
+	xfree(to_quoted);
+
+	return (dest);
 }
 
 char *
@@ -226,9 +269,7 @@ buildCommandString(const char *extension, const char *fileName,
 	char	*encoder = NULL;
 	char	*decoder = NULL;
 	char	*newDecoder = NULL;
-	size_t	 newDecoderLen = 0;
 	char	*newEncoder = NULL;
-	size_t	 newEncoderLen = 0;
 	char	*localTitle = UTF8toCHAR(metadata_get_title(mdata),
 					 ICONV_REPLACE);
 	char	*localArtist = UTF8toCHAR(metadata_get_artist(mdata),
@@ -246,23 +287,16 @@ buildCommandString(const char *extension, const char *fileName,
 		xfree(decoder);
 		return (NULL);
 	}
-	newDecoderLen = strlen(decoder) + strlen(fileName) + 1;
-	newDecoder = xcalloc(newDecoderLen, sizeof(char));
-	replaceString(decoder, newDecoder, newDecoderLen, TRACK_PLACEHOLDER,
-		      fileName);
+	newDecoder = replaceString(decoder, TRACK_PLACEHOLDER, fileName);
 	if (strstr(decoder, ARTIST_PLACEHOLDER) != NULL) {
-		size_t tmpLen = strlen(newDecoder) + strlen(localArtist) + 1;
-		char *tmpStr = xcalloc(tmpLen, sizeof(char));
-		replaceString(newDecoder, tmpStr, tmpLen, ARTIST_PLACEHOLDER,
-			      localArtist);
+		char *tmpStr = replaceString(newDecoder, ARTIST_PLACEHOLDER,
+		    localArtist);
 		xfree(newDecoder);
 		newDecoder = tmpStr;
 	}
 	if (strstr(decoder, TITLE_PLACEHOLDER) != NULL) {
-		size_t tmpLen = strlen(newDecoder) + strlen(localTitle) + 1;
-		char *tmpStr = xcalloc(tmpLen, sizeof(char));
-		replaceString(newDecoder, tmpStr, tmpLen, TITLE_PLACEHOLDER,
-			      localTitle);
+		char *tmpStr = replaceString(newDecoder, TITLE_PLACEHOLDER,
+		    localTitle);
 		xfree(newDecoder);
 		newDecoder = tmpStr;
 	}
@@ -279,27 +313,20 @@ buildCommandString(const char *extension, const char *fileName,
 	if (strstr(decoder, METADATA_PLACEHOLDER) != NULL) {
 		if (metadataFromProgram && pezConfig->metadataFormat != NULL) {
 			char *mdataString = getMetadataString(pezConfig->metadataFormat, mdata);
-			size_t tmpLen = strlen(newDecoder) + strlen(mdataString) + 1;
-			char *tmpStr = xcalloc(tmpLen, sizeof(char));
-			replaceString(newDecoder, tmpStr, tmpLen,
-				      METADATA_PLACEHOLDER, mdataString);
+			char *tmpStr = replaceString(newDecoder,
+			    METADATA_PLACEHOLDER, mdataString);
 			xfree(newDecoder);
 			xfree(mdataString);
 			newDecoder = tmpStr;
 		} else {
 			if (!metadataFromProgram && strstr(decoder, TITLE_PLACEHOLDER) != NULL) {
-				size_t tmpLen = strlen(newDecoder) + 1;
-				char *tmpStr = xcalloc(tmpLen, sizeof(char));
-				replaceString(newDecoder, tmpStr, tmpLen,
-					      METADATA_PLACEHOLDER, "");
+				char *tmpStr = replaceString(newDecoder,
+				    METADATA_PLACEHOLDER, "");
 				xfree(newDecoder);
 				newDecoder = tmpStr;
 			} else {
-				size_t tmpLen = strlen(newDecoder) + strlen(localMetaString) + 1;
-				char *tmpStr = xcalloc(tmpLen, sizeof(char));
-				replaceString(newDecoder, tmpStr, tmpLen,
-					      METADATA_PLACEHOLDER,
-					      localMetaString);
+				char *tmpStr = replaceString(newDecoder,
+				    METADATA_PLACEHOLDER, localMetaString);
 				xfree(newDecoder);
 				newDecoder = tmpStr;
 			}
@@ -325,42 +352,30 @@ buildCommandString(const char *extension, const char *fileName,
 		return (commandString);
 	}
 
-	newEncoderLen = strlen(encoder) + strlen(localArtist) + 1;
-	newEncoder = xcalloc(newEncoderLen, sizeof(char));
-	replaceString(encoder, newEncoder, newEncoderLen, ARTIST_PLACEHOLDER,
-		      localArtist);
+	newEncoder = replaceString(encoder, ARTIST_PLACEHOLDER, localArtist);
 	if (strstr(encoder, TITLE_PLACEHOLDER) != NULL) {
-		size_t tmpLen = strlen(newEncoder) + strlen(localTitle) + 1;
-		char *tmpStr = xcalloc(tmpLen, sizeof(char));
-		replaceString(newEncoder, tmpStr, tmpLen, TITLE_PLACEHOLDER,
-			      localTitle);
+		char *tmpStr = replaceString(newEncoder, TITLE_PLACEHOLDER,
+		    localTitle);
 		xfree(newEncoder);
 		newEncoder = tmpStr;
 	}
 	if (strstr(encoder, METADATA_PLACEHOLDER) != NULL) {
 		if (metadataFromProgram && pezConfig->metadataFormat != NULL) {
 			char *mdataString = getMetadataString(pezConfig->metadataFormat, mdata);
-			size_t tmpLen = strlen(newEncoder) + strlen(mdataString) + 1;
-			char *tmpStr = xcalloc(tmpLen, sizeof(char));
-			replaceString(newEncoder, tmpStr, tmpLen,
-				      METADATA_PLACEHOLDER, mdataString);
+			char *tmpStr = replaceString(newEncoder,
+			    METADATA_PLACEHOLDER, mdataString);
 			xfree(newEncoder);
 			xfree(mdataString);
 			newEncoder = tmpStr;
 		} else {
 			if (!metadataFromProgram && strstr(encoder, TITLE_PLACEHOLDER) != NULL) {
-				size_t tmpLen = strlen(newEncoder) + 1;
-				char *tmpStr = xcalloc(tmpLen, sizeof(char));
-				replaceString(newEncoder, tmpStr, tmpLen,
-					      METADATA_PLACEHOLDER, "");
+				char *tmpStr = replaceString(newEncoder,
+				    METADATA_PLACEHOLDER, "");
 				xfree(newEncoder);
 				newEncoder = tmpStr;
 			} else {
-				size_t tmpLen = strlen(newEncoder) + strlen(localMetaString) + 1;
-				char *tmpStr = xcalloc(tmpLen, sizeof(char));
-				replaceString(newEncoder, tmpStr, tmpLen,
-					      METADATA_PLACEHOLDER,
-					      localMetaString);
+				char *tmpStr = replaceString(newEncoder,
+				    METADATA_PLACEHOLDER, localMetaString);
 				xfree(newEncoder);
 				newEncoder = tmpStr;
 			}
@@ -388,7 +403,6 @@ char *
 getMetadataString(const char *format, metadata_t *mdata)
 {
 	char	*tmp, *str;
-	size_t	 len;
 
 	if (mdata == NULL) {
 		printf("%s: getMetadataString(): Internal error: NULL metadata_t\n",
@@ -402,34 +416,26 @@ getMetadataString(const char *format, metadata_t *mdata)
 	str = xstrdup(format);
 
 	if (strstr(format, ARTIST_PLACEHOLDER) != NULL) {
-		len = strlen(str) + strlen(metadata_get_artist(mdata)) + 1;
-		tmp = xcalloc(len, sizeof(char));
-		replaceString(str, tmp, len, ARTIST_PLACEHOLDER,
-			      metadata_get_artist(mdata));
+		tmp = replaceString(str, ARTIST_PLACEHOLDER,
+		    metadata_get_artist(mdata));
 		xfree(str);
 		str = tmp;
 	}
 	if (strstr(format, TITLE_PLACEHOLDER) != NULL) {
-		len = strlen(str) + strlen(metadata_get_title(mdata)) + 1;
-		tmp = xcalloc(len, sizeof(char));
-		replaceString(str, tmp, len, TITLE_PLACEHOLDER,
-			      metadata_get_title(mdata));
+		tmp = replaceString(str, TITLE_PLACEHOLDER,
+		    metadata_get_title(mdata));
 		xfree(str);
 		str = tmp;
 	}
 	if (strstr(format, STRING_PLACEHOLDER) != NULL) {
-		len = strlen(str) + strlen(metadata_get_string(mdata)) + 1;
-		tmp = xcalloc(len, sizeof(char));
-		replaceString(str, tmp, len, STRING_PLACEHOLDER,
-			      metadata_get_string(mdata));
+		tmp = replaceString(str, STRING_PLACEHOLDER,
+		    metadata_get_string(mdata));
 		xfree(str);
 		str = tmp;
 	}
 	if (strstr(format, TRACK_PLACEHOLDER) != NULL) {
-		len = strlen(str) + strlen(metadata_get_filename(mdata)) + 1;
-		tmp = xcalloc(len, sizeof(char));
-		replaceString(str, tmp, len, TRACK_PLACEHOLDER,
-			      metadata_get_filename(mdata));
+		tmp = replaceString(str, TRACK_PLACEHOLDER,
+		    metadata_get_filename(mdata));
 		xfree(str);
 		str = tmp;
 	}
@@ -477,6 +483,9 @@ setMetadata(shout_t *shout, metadata_t *mdata, char **mdata_copy)
 		abort();
 	}
 
+	if (mFlag)
+		return (SHOUTERR_SUCCESS);
+
 	if (mdata == NULL)
 		return 1;
 
@@ -750,6 +759,7 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 	int		 ret;
 	double		 kbps = -1.0;
 	struct timeval	 timeStamp, *startTime = tv;
+	struct timeval	 callTime, currentTime;
 
 	if (startTime == NULL) {
 		printf("%s: sendStream(): Internal error: startTime is NULL\n",
@@ -757,6 +767,8 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 		abort();
 	}
 
+	ez_gettimeofday((void *)&callTime);
+
 	timeStamp.tv_sec = startTime->tv_sec;
 	timeStamp.tv_usec = startTime->tv_usec;
 
@@ -795,7 +807,15 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 			ret = STREAM_SKIP;
 			break;
 		}
-		if (queryMetadata) {
+
+		ez_gettimeofday((void *)&currentTime);
+
+		if (queryMetadata ||
+		    (pezConfig->metadataRefreshInterval != -1
+		     && (currentTime.tv_sec - callTime.tv_sec
+			 >= pezConfig->metadataRefreshInterval)
+		    )
+		   ) {
 			queryMetadata = 0;
 			if (metadataFromProgram) {
 				ret = STREAM_UPDMDATA;
@@ -805,8 +825,7 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 
 		total += bytes_read;
 		if (qFlag && vFlag) {
-			struct timeval	tval;
-			double		oldTime, newTime;
+			double	oldTime, newTime;
 
 			if (!isStdin && playlistMode) {
 				if (pezConfig->fileNameIsProgram) {
@@ -822,20 +841,22 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 
 			oldTime = (double)timeStamp.tv_sec
 				+ (double)timeStamp.tv_usec / 1000000.0;
-			ez_gettimeofday((void *)&tval);
-			newTime = (double)tval.tv_sec
-				+ (double)tval.tv_usec / 1000000.0;
+			newTime = (double)currentTime.tv_sec
+				+ (double)currentTime.tv_usec / 1000000.0;
 			if (songLenStr == NULL)
 				printf("  [ %s]",
-				       getTimeString(tval.tv_sec - startTime->tv_sec));
+				       getTimeString(currentTime.tv_sec -
+						     startTime->tv_sec));
 			else
 				printf("  [ %s/%s]",
-				       getTimeString(tval.tv_sec - startTime->tv_sec),
+				       getTimeString(currentTime.tv_sec -
+						     startTime->tv_sec),
 				       songLenStr);
 			if (newTime - oldTime >= 1.0) {
-				kbps = (((double)(total - oldTotal) / (newTime - oldTime)) * 8.0) / 1000.0;
-				timeStamp.tv_sec = tval.tv_sec;
-				timeStamp.tv_usec = tval.tv_usec;
+				kbps = (((double)(total - oldTotal)
+					 / (newTime - oldTime)) * 8.0) / 1000.0;
+				timeStamp.tv_sec = currentTime.tv_sec;
+				timeStamp.tv_usec = currentTime.tv_usec;
 				oldTotal = total;
 			}
 			if (kbps < 0)
@@ -936,6 +957,8 @@ streamFile(shout_t *shout, const char *fileName)
 			}
 			if (ret == STREAM_UPDMDATA || queryMetadata) {
 				queryMetadata = 0;
+				if (mFlag)
+					continue;
 				if (metadataFromProgram) {
 					char		*mdataStr = NULL;
 					metadata_t	*prog_mdata;
@@ -954,8 +977,9 @@ streamFile(shout_t *shout, const char *fileName)
 						continue;
 					}
 					metadata_free(&prog_mdata);
-					printf("%s: New metadata: ``%s''\n",
-					       __progname, mdataStr);
+					if (vFlag > 1)
+						printf("%s: New metadata: ``%s''\n",
+						       __progname, mdataStr);
 					xfree(mdataStr);
 				}
 			}
@@ -969,7 +993,7 @@ streamFile(shout_t *shout, const char *fileName)
 
 	if (popenFlag)
 		pclose(filepstream);
-	else
+	else if (!isStdin)
 		fclose(filepstream);
 
 	if (songLenStr != NULL)
@@ -1073,7 +1097,7 @@ ez_shutdown(int exitval)
 void
 usage(void)
 {
-	printf("usage: %s [-hnqVv] -c configfile\n", __progname);
+	printf("usage: %s [-hmnqVv] -c configfile\n", __progname);
 	printf("       %s -s [playlist]\n", __progname);
 }
 
@@ -1083,6 +1107,7 @@ usageHelp(void)
 	printf("\n");
 	printf("  -c configfile  use XML configuration in configfile (mandatory)\n");
 	printf("  -h             display this additional help and exit\n");
+	printf("  -m             disable metadata updates\n");
 	printf("  -n             normalize metadata strings\n");
 	printf("  -q             suppress STDERR output from external en-/decoders\n");
 	printf("  -s [playlist]  read lines from playlist (or STDIN), shuffle and print them to\n");
@@ -1120,11 +1145,12 @@ main(int argc, char *argv[])
 	__progname = getProgname(argv[0]);
 	pezConfig = getEZConfig();
 
+	mFlag = 0;
 	nFlag = 0;
 	qFlag = 0;
 	vFlag = 0;
 
-	while ((c = local_getopt(argc, argv, "c:hnqsVv")) != -1) {
+	while ((c = local_getopt(argc, argv, "c:hmnqsVv")) != -1) {
 		switch (c) {
 		case 'c':
 			if (configFile != NULL) {
@@ -1138,6 +1164,9 @@ main(int argc, char *argv[])
 			usage();
 			usageHelp();
 			return (ez_shutdown(0));
+		case 'm':
+			mFlag = 1;
+			break;
 		case 'n':
 			nFlag = 1;
 			break;
@@ -1316,7 +1345,6 @@ main(int argc, char *argv[])
 			printf("%s: Using program '%s' to get filenames for streaming\n",
 			       __progname, pezConfig->fileName);
 
-		ret = 1;
 		do {
 			if (playlistMode) {
 				ret = streamPlaylist(shout,
